/* ====================================================================
 * Copyright (c) 2000 Dean Gaudet.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dean Gaudet"
 *
 * THIS SOFTWARE IS PROVIDED BY DEAN GAUDET ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DEAN GAUDET OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
#ifndef SPLIM_KEV_H
#define SPLIM_KEV_H 1

/* kev is kernel event
 * 
 * the events the kernel provides us with are
 * i/o ready indications on file descriptors
 * and timers
 *
 * this is merely a wrapper around the poll() interface
 *
 * events only trigger once -- they must be re-registered
 * if multiple triggers are desired
 */

#include <sys/poll.h>

/* kev_io_t is an io event -- triggered by the poll flags */
typedef struct kev_io_t kev_io_t;

/* kev_timeout_t is a timeout event */
typedef struct kev_timeout_t kev_timeout_t;

extern kev_io_t *kev_register_io(splim_t *, int fd, int events, void *uap, void (*callback)(void *uap, int revents));
extern void kev_unregister_io(kev_io_t *);

extern kev_timeout_t *kev_register_timeout(splim_t *, int microseconds, void *uap, void (*callback)(void *uap));
extern void kev_unregister_timeout(kev_timeout_t *);

extern int kev_main_loop(splim_t *);

#endif
