/* ====================================================================
 * Copyright (c) 2000 Dean Gaudet.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dean Gaudet"
 *
 * THIS SOFTWARE IS PROVIDED BY DEAN GAUDET ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DEAN GAUDET OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
#ifndef SPLIM_KEV_P_H
#define SPLIM_KEV_P_H 1

struct kev_io_t {
	kev_io_t *io_next;
	kev_io_t *io_prev;
	int fd;
	int events;
	void *uap;
	/* callback set to NULL indicates a pending cleanup */
	void (*callback)(void *uap, int revents);
};

ring_template(kev_io_t, io)

/* number of microseconds since 00:00:00 january 1, 1970 UTC */
typedef int64_t kev_time_t;

struct kev_timeout_t {
	kev_timeout_t *timeout_next;
	kev_timeout_t *timeout_prev;
	kev_time_t trigger_time;
	void *uap;
	/* callback set to NULL indicates a pending cleanup */
	void (*callback)(void *uap);
};

ring_template(kev_timeout_t, timeout)


typedef struct {
	io_ring_t ios;
	timeout_ring_t timeouts;
	kev_time_t last_event;
	pezz_t *io_pezz;
	pezz_t *timeout_pezz;
} kev_ctx_t;

extern void kev_ctx_init(splim_t *);
extern void kev_ctx_fini(splim_t *);

#endif
