/* ====================================================================
 * Copyright (c) 2000 Dean Gaudet.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dean Gaudet"
 *
 * THIS SOFTWARE IS PROVIDED BY DEAN GAUDET ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DEAN GAUDET OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
#ifndef SPLIM_PEZZ_H
#define SPLIM_PEZZ_H 1

/*
	thank jasone for the name pezz... the concept though is
	pretty old.  a pezz is an allocator for a fixed-sized object.
	it allocates arrays of the objects to reduce malloc overhead;
	and frees back to a stack.  the result is L1/L2-cache goodness.
*/

#include <stddef.h>

typedef struct pezz_t pezz_t;

extern pezz_t *pezz_create(size_t element_size, size_t dispenser_size);
extern void pezz_destroy(pezz_t *);
extern void *pezz_alloc(pezz_t *);
extern void pezz_free(pezz_t *, void *);

/*
i'm not really sure what makes a pezz happy, but i think 128 will do it.
*/
#define HAPPY_PEZZ (128)

/* is the supplied pointer a member of this pezz? 
 * useful for debugging
 */
extern int pezz_is_member_of(pezz_t *, void *);

/* uap is a pezz_t *, and we will call pezz_free(uap, buf) */
extern void pezz_bufc_free(void *uap, void *buf);

#endif
