/* ====================================================================
 * Copyright (c) 2000 Dean Gaudet.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dean Gaudet"
 *
 * THIS SOFTWARE IS PROVIDED BY DEAN GAUDET ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DEAN GAUDET OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
#ifndef SPLIM_TLA_H
#define SPLIM_TLA_H 1

/* TLA = Transport Layer Abstraction.  heh. */

/* hrm, really need to figure out this address thing... */
#include <netinet/in.h>

typedef struct tla_t tla_t;
typedef void *tla_address_t;

typedef struct {
	/* create a new vfd which is bound to the specified address.
	 * the vfd accept() method is used to accept incoming connections.
	 * the vfd may be selected for read to indicate when it has incoming
	 * connections to be accepted.
	 */
	vfd_t *(*bind)(splim_t *ctx, tla_address_t);

	/* initiate a connection to the specified address, and return a
	 * vfd.  the caller should then select the vfd for write.
	 * when the write callback is called, the caller should
	 * invoke vfd_getsockopt(VFD_SOCKERR) to see if any error
	 * occured during the connect.
	 */
	vfd_t *(*connect)(splim_t *ctx, tla_address_t);
} tla_methods_t;

extern const tla_methods_t TCP;
extern const tla_methods_t SSL;
extern const tla_methods_t SCP;

#endif
