/* ====================================================================
 * Copyright (c) 2000 Dean Gaudet.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dean Gaudet"
 *
 * THIS SOFTWARE IS PROVIDED BY DEAN GAUDET ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DEAN GAUDET OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
#include <splim/splim.h>
#include <openssl/ssl.h>

#include "splim/splim_p.h"

struct ssl_data_t {
	pezz_t *vfd_ssl_pezz;
	SSL_CTX *ssl_ctx;
	RSA *tmp_rsa512;
};

typedef struct {
	vfd_t vfd;
	vfd_t *transport;
} vfd_ssl_t;

static RSA *tmp_rsa512;


static vfd_t *ssl_bind(splim_t *ctx, tla_address_t vaddr)
{
	vfd_t *transport;
	vfd_ssl_t *vfd;

	transport = TCP.bind(ctx, vaddr);
	if (!transport) {
		return NULL;
	}
	vfd = pezz_alloc(ctx->ssl.vfd_ssl_pezz);
	vfd->transport = transport;
}


static vfd_t *ssl_connect(tla_t *tla, tla_address_t vaddr)
{
}


static RSA MS_CALLBACK *tmp_rsa_cb(SSL *s, int is_export, int keylength)
{
	return tmp_rsa512;
}


void ssl_init(splim_t *ctx)
{
	ctx->ssl.vfd_ssl_pezz = pezz_create(sizeof(vfd_ssl_t), HAPPY_PEZZ);

	SSL_load_error_strings();
	SSL_library_init();
	X509V3_add_standard_extensions();

	ctx->ssl.ssl_ctx = SSL_CTX_new(SSLv23_method());
	if (ctx->ssl.ssl_ctx == NULL) {
		die();
	}

	SSL_CTX_set_quiet_shutdown(ctx->ssl.ssl_ctx, 1);
	SSL_CTX_set_options(ctx->ssl.ssl_ctx, 0);

	/* XXX: hmm, how does session cache work? */
	SSL_CTX_sess_set_cache_size(ctx,128);

	/* XXX: need to initialise random pool?  i think it gets /dev/urandom by default on linux/freebsd */

	/* XXX: should regenerate tmp RSA key on -HUP */
	tmp_rsa512 = RSA_generate_key(512, RSA_F4, NULL, NULL);
	if (tmp_rsa512 == NULL) {
		syslog(LOG_ERR, "unable to generate 512-bit RSA export key");
		ERR_print_errors(bio_syslog);
		exit(1);
	}
	SSL_CTX_set_tmp_rsa_callback(ctx,tmp_rsa_cb);

}
