/* ====================================================================
 * Copyright (c) 2000 Dean Gaudet.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dean Gaudet"
 *
 * THIS SOFTWARE IS PROVIDED BY DEAN GAUDET ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DEAN GAUDET OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
#include <splim/splim.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <netinet/tcp.h>

#include "splim/splim_p.h"


static vfd_t *tcp_bind(splim_t *ctx, tla_address_t vaddr)
{
	struct sockaddr_in *addr = vaddr;
	int s;
	int one;
	vfd_t *vfd;
	int save_errno;

	s = socket(AF_INET, SOCK_STREAM, 0);
	if (s < 0) {
		return NULL;
	}

	one = 1;
	if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one)) < 0) {
doom:
		save_errno = errno;
		close(s);
		errno = save_errno;
		return NULL;
	}

	if (bind(s, (struct sockaddr *)addr, sizeof(*addr)) < 0) {
		goto doom;
	}

	if (listen(s, 511) < 0) {
		goto doom;
	}

	if (fcntl(s, F_SETFL, O_RDWR | O_NONBLOCK) < 0) {
		goto doom;
	}

	vfd = vfd_socket(ctx, s);
	if (!vfd) {
		close(s);
		return NULL;
	}
	return vfd;
}


static vfd_t *tcp_connect(splim_t *ctx, tla_address_t vaddr)
{
	struct sockaddr_in *dest = vaddr;
	int s;
	int save_errno;
	vfd_t *vfd;
	int just_say_no = 1;

	/* create socket */
	s = socket(AF_INET, SOCK_STREAM, 0);
	if (s < 0) {
		return NULL;
	}

	/* mark non-blocking */
	if (fcntl(s, F_SETFL, O_RDWR | O_NONBLOCK) < 0) {
doom:
		save_errno = errno;
		close(s);
		errno = save_errno;
		return NULL;
	}

	/* initiate the connect */
	if (connect(s, (struct sockaddr *)dest, sizeof(struct sockaddr_in)) < 0) {
		if (errno != EINPROGRESS) {
			goto doom;
		}
	}

	/* disable nagle */
	if (setsockopt(s, IPPROTO_TCP, TCP_NODELAY, (char *)&just_say_no, sizeof(just_say_no)) < 0) {
		goto doom;
	}

	/* create the vfd */
	vfd = vfd_socket(ctx, s);
	if (!vfd) {
		goto doom;
	}

	return vfd;
}


const tla_methods_t TCP = {
	.bind = tcp_bind,
	.connect = tcp_connect,
};
